#ifndef EAVESDROPPER_ABSTRACTNODEVIEW_H
#define EAVESDROPPER_ABSTRACTNODEVIEW_H

// ***************************************************************************
// System Includes
// ***************************************************************************

#include <be/interface/Point.h>
#include <be/interface/View.h>

#include <be/support/List.h>

// ***************************************************************************
// Project Includes
// ***************************************************************************

#include "StyleGuideDefs.h"         // Standard type and constant definitions.

// ***************************************************************************
// External Name Declarations
// ***************************************************************************

class InfoWindow;

// ***************************************************************************
// External Variable Declarations
// ***************************************************************************

// ***************************************************************************
// Constant Definitions
// ***************************************************************************

// ***************************************************************************
// Type Definitions
// ***************************************************************************

// ***************************************************************************
// Class Definition
// ***************************************************************************

/**
 * The AbstractNodeView class defines the common behaviors for the
 * LiveNodeView and DormantNodeView classes.
 */
class AbstractNodeView : public BView
{
public:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	// Attribute Definitions =================================================

	// Member Function Definitions ===========================================

	/**
	 * Constructor.
	 *
	 * @param infoWindow Pointer to the Information window.
	 * @param nodeName The name of the node.
	 */
	AbstractNodeView(InfoWindow* infoWindow, const char* nodeName);

	/** Automatically generated copy constructor. */
	// LiveAbstractNodeView(const AbstractNodeView& old);

	/** Destructor. */
	virtual ~AbstractNodeView(void);

	/** Automatically generated assignment operator. */
	// AbstractNodeView& operator=(AbstractNodeView& right);

	/** Tells the LiveNodesView to select this node. */
	virtual void MouseDown(BPoint where);

	/** Figures out whether the mouse is currently rolling over the view. */
	virtual void MouseMoved(BPoint point,
		uint32 transit,
		const BMessage* message);
	
	/** Selects or de-selects this node. */
	void SetSelected(bool flag);

	/** Determines whether this node is selected. */
	bool IsSelected();

protected:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	// Attribute Definitions =================================================

	/** The InfoWindow object that does the selections. */
	InfoWindow* infoWindow;

	/** The list that stores the different chunks of the node's name. */
	BList* nameChunks;

	/** Whether this node is currently selected. */
	bool isSelected;

	/** Whether the mouse is currently hovering over this node. */
	bool isRollover;		

	/** The height of the view's font. */
	font_height fontHeight;
	
	/** Sum total of fontHeight's ascent, descent, and leading. */
	float fontSize;

	// Member Function Definitions ===========================================

	/**
	 * Draws the node. You should call DrawNode() from the overriding
	 * class's Draw() method.
	 */
	void DrawNode(int32 nodeId, int32 nodeKinds);
	
	/**
	 * Splits the node's name into chunks and stores them into the
	 * nameChunks list.
	 */
	void SplitUpName(const char* nodeName);
	
	/**
	 * Truncates a string to the node's width and adds it to the nameChunks
	 * list.
	 */
	void AddNameChunk(const char* chunk);

private:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	// Attribute Definitions =================================================

	// Member Function Definitions ===========================================
};

// ***************************************************************************
// Global Variable Declarations
// ***************************************************************************

// ***************************************************************************
// Inline Function Definitions
// ***************************************************************************

// ***************************************************************************
// Helper Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Function Declarations
// ***************************************************************************

#endif // EAVESDROPPER_ABSTRACTNODEVIEW_H
