// ***************************************************************************
// Implementation System Includes
// ***************************************************************************

// ***************************************************************************
// Implementation Project Includes
// ***************************************************************************

#include "ConnectorView.h"
#include "EavesdropperMain.h"
#include "InfoWindow.h"

// ***************************************************************************
// Implementation Constant Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Type Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Variable Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Variable Definitions
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions
// ***************************************************************************

// ===========================================================================
// Constructor
// ===========================================================================

ConnectorView::ConnectorView(
		InfoWindow* infoWindow, 
		media_input* mediaInput,
		media_output* mediaOutput,
		bool isConnected)
	: BView(BRect(0, 0, 8, 8),
		"ConnectorView",
		B_FOLLOW_NONE,
		B_WILL_DRAW) 
{
	// Initialize attributes.
	this->infoWindow = infoWindow;
	this->mediaInput = mediaInput;
	this->mediaOutput = mediaOutput;
	this->isConnected = isConnected;
	isSelected = false;
	isRollover = false;
	
	SetViewColor(COLOR_BACKGROUND);
}

// ===========================================================================
// Destructor
// ===========================================================================

ConnectorView::~ConnectorView(void)
{
	// Do nothing.
}

// ===========================================================================
// MessageReceived
// ===========================================================================

void
ConnectorView::MessageReceived(BMessage* message)
{
	// Figure out what kind of message was received.
	switch(message->what)
	{
		// Let the original BView handle the message.
		default:
		{
			BView::MessageReceived(message);
			break;
		}
	}
}

// ===========================================================================
// Draw
// ===========================================================================

void 
ConnectorView::Draw(BRect updateRect)
{
	BView::Draw(updateRect);
	
	SetDrawingMode(B_OP_OVER);

	BRect bounds = Bounds();
	float width  = bounds.Width();
	float height = bounds.Height();

	if (isSelected)
	{
		SetHighColor(COLOR_SELECTED);
		FillEllipse(BPoint(width/2,height/2), width/2, height/2);
	}
	else if (isRollover)
	{
		SetHighColor(COLOR_ROLLOVER);
		FillEllipse(BPoint(width/2,height/2), width/2, height/2);
	}

	if (isConnected)
	{	
		SetHighColor(COLOR_CONNECTED);
	}
	else
	{
			SetHighColor(COLOR_FREE);
	}
	FillEllipse(BPoint(width/2,height/2), width/2-1, height/2-1);

	SetHighColor(COLOR_BORDER);
	StrokeEllipse(BPoint(width/2,height/2), width/2-1, height/2-1);
}

// ===========================================================================
// MouseDown
// ===========================================================================

void
ConnectorView::MouseDown(BPoint where)
{
	// Ask the live nodes view to select us.
	if (!isSelected)
	{
		infoWindow->SelectConnector(this);
	}
}

// ===========================================================================
// MouseMoved
// ===========================================================================

void
ConnectorView::MouseMoved(BPoint point,
	uint32 transit,
	const BMessage* message)
{
	// Don't put the node in the rollover state if the user is dragging
	// another node on top of us.
	if (message != 0)
	{
		return;
	}

	if (transit == B_ENTERED_VIEW)
	{
		isRollover = true;

		if (!isSelected)
		{
			Invalidate();
		}
	}
	else if (transit == B_EXITED_VIEW)
	{
		isRollover = false;

		if (!isSelected)
		{
			Invalidate();
		}
	}
}

// ===========================================================================
// SetSelected
// ===========================================================================

void
ConnectorView::SetSelected(bool flag)
{
	isSelected = flag;
	Invalidate();
}

// ===========================================================================
// IsSelected
// ===========================================================================

bool
ConnectorView::IsSelected()
{
	return isSelected;
}

// ===========================================================================
// IsInput
// ===========================================================================

bool
ConnectorView::IsInput()
{
	return (mediaInput != 0);
}

// ===========================================================================
// IsOutput
// ===========================================================================

bool
ConnectorView::IsOutput()
{
	return (mediaOutput != 0);
}

// ===========================================================================
// GetInput
// ===========================================================================

media_input*
ConnectorView::GetInput()
{
	return mediaInput;
}

// ===========================================================================
// GetOutput
// ===========================================================================

media_output*
ConnectorView::GetOutput()
{
	return mediaOutput;
}

// ===========================================================================
// IsConnected
// ===========================================================================

bool
ConnectorView::IsConnected()
{
	return isConnected;
}
	
// ***************************************************************************
// PROTECTED Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// PRIVATE Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// Helper Function Definitions
// ***************************************************************************

// ***************************************************************************
// Global Function Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Definitions
// ***************************************************************************
