// ***************************************************************************
// Implementation System Includes
// ***************************************************************************

#include <be/interface/Alert.h>
#include <be/media/MediaRoster.h>
#include <be/storage/Path.h>

#include <posix/unistd.h>			// chdir()

// ***************************************************************************
// Implementation Project Includes
// ***************************************************************************

#include "EavesdropperApp.h"
#include "EavesdropperMain.h"
#include "Fail.h"
#include "IconFactory.h"
#include "InfoWindow.h"
#include "MainWindow.h"
#include "Version.h"

// ***************************************************************************
// Implementation Constant Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Type Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Variable Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Variable Definitions
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions
// ***************************************************************************

// ===========================================================================
// Constructor
// ===========================================================================

EavesdropperApp::EavesdropperApp(void) 
	: BApplication("application/x-vnd.WeirdExperiments.Eavesdropper")
{  
	// Set the current working directory to where the application lives.
	app_info info; 
	be_app->GetAppInfo(&info); 
	BEntry entry(&info.ref); 
	BPath path; 
	entry.GetPath(&path); 
	path.GetParent(&path); 
	chdir(path.Path()); 

	// -----------------------------------------------------------------------

	// Get the Media Roster.
	status_t result; 
	BMediaRoster* mediaRoster = BMediaRoster::Roster(&result);

	// Check if successful.
	if ((mediaRoster == 0) || (result != B_OK))
	{ 
		BBG_WRITE(("[EavesdropperApp::EavesdropperApp] Cannot connect to"
			" the Media Server. (%s)", strerror(result)));
               
		Fail("Sorry, but I cannot connect to the Media Server."
			" Please make sure that it is running and try again.");

		PostMessage(B_QUIT_REQUESTED);
		return;
	} 

	// Successfully established a connection to the Media Server.
	BBG_WRITE(("[EavesdropperApp::EavesdropperApp] Connected to the"
		" Media Server."));

	// -----------------------------------------------------------------------

	// Initialize the IconFactory.
	IconFactory::Initialize();

	// Create the Information window.
	InfoWindow* infoWindow = new InfoWindow(mediaRoster);
	
	// Create the main window.
	MainWindow* mainWindow = new MainWindow(mediaRoster, infoWindow);
	
	// And show the windows.
	mainWindow->Show();
	infoWindow->Show();
}

// ===========================================================================
// MessageReceived
// ===========================================================================

void
EavesdropperApp::MessageReceived(BMessage* message)
{
	// Figure out what kind of message was received.
	switch(message->what)
	{
		// Let the original BApplication handle the message.
		default:
		{
			BApplication::MessageReceived(message);
			break;
		}
	}
}

// ===========================================================================
// AboutRequested
// ===========================================================================

void
EavesdropperApp::AboutRequested(void)
{
	(new BAlert("About",
		"Eavesdropper\n\n"
		"Version " VERSION_STRING "\n"
		"Released on " VERSION_DATE_STRING "\n\n"
		"Created by Matthijs Hollemans\n"
		"matthijs@weirdexperiments.com\n"
		"http://www.weirdexperiments.com\n\n"
		"This version of Eavesdropper is public domain, so you may do"
		" anything you like with it. No warranties expressed or implied.",
		"OK"))->Go(0);
}

// ***************************************************************************
// PROTECTED Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// PRIVATE Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// Helper Function Definitions
// ***************************************************************************

// ***************************************************************************
// Global Function Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Definitions
// ***************************************************************************
