#ifndef EAVESDROPPER_ICONFACTORY_H
#define EAVESDROPPER_ICONFACTORY_H

// ***************************************************************************
// System Includes
// ***************************************************************************

#include <be/interface/Bitmap.h>

// ***************************************************************************
// Project Includes
// ***************************************************************************

// Standard type and constant definitions.
#include "StyleGuideDefs.h"

// ***************************************************************************
// External Name Declarations
// ***************************************************************************

// ***************************************************************************
// External Variable Declarations
// ***************************************************************************

// ***************************************************************************
// Constant Definitions
// ***************************************************************************

// ***************************************************************************
// Type Definitions
// ***************************************************************************

// ***************************************************************************
// Class Definition
// ***************************************************************************

/** 
 * The IconFactory class provides easy access to the various icons that
 * Eavesdropper uses.
 *
 * You must call IconFactory::Initialize() before you may use this class.
 */
class IconFactory 
{
public:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	/** The different kinds of icons. */
	enum Icon
	{
		BUFFER_PRODUCER = 0,
		BUFFER_CONSUMER,
		TIME_SOURCE,
		CONTROLLABLE,
		FILE_INTERFACE,
		PHYSICAL_INPUT,
		PHYSICAL_OUTPUT,
		SYSTEM_MIXER,
	};
	
	// Attribute Definitions =================================================

	// Member Function Definitions ===========================================

	/** Automatically generated copy constructor. */
	// IconFactory(const IconFactory& old);

	/** Automatically generated destructor. */
	// virtual ~IconFactory(void);

	/** Automatically generated assignment operator. */
	// IconFactory& operator=(IconFactory& right);

	/** Must be called before IconFactory may be used. */
	static void Initialize();

	/** Returns the BBitmap for a particular icon. */
	static BBitmap* GetBitmapFor(Icon icon);

protected:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	// Attribute Definitions =================================================

	// Member Function Definitions ===========================================

private:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	// Attribute Definitions =================================================

	/** The bitmap for the buffer producer icon. */
	static BBitmap* bufferProducerBitmap;

	/** The bitmap for the buffer consumer icon. */
	static BBitmap* bufferConsumerBitmap;

	/** The bitmap for the time source icon. */
	static BBitmap* timeSourceBitmap;

	/** The bitmap for the controllable icon. */
	static BBitmap* controllableBitmap;

	/** The bitmap for the file interface icon. */
	static BBitmap* fileInterfaceBitmap;

	/** The bitmap for the physical input icon. */
	static BBitmap* physicalInputBitmap;

	/** The bitmap for the physical output icon. */
	static BBitmap* physicalOutputBitmap;
		
	/** The bitmap for the system mixer icon. */
	static BBitmap* systemMixerBitmap;

	// Member Function Definitions ===========================================

	/** The constructor is disabled. */
	IconFactory(void);
};

// ***************************************************************************
// Global Variable Declarations
// ***************************************************************************

// ***************************************************************************
// Inline Function Definitions
// ***************************************************************************

// ***************************************************************************
// Helper Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Function Declarations
// ***************************************************************************

#endif // EAVESDROPPER_ICONFACTORY_H
