/*!
 * [TODO] Create a Settings class that can load and save itself.
 */

// ***************************************************************************
// Implementation System Includes
// ***************************************************************************

#include <be/interface/ScrollView.h>

#include <be/storage/Entry.h>
#include <be/storage/Mime.h>
#include <be/storage/Path.h>

#include <posix/stdio.h>

// ***************************************************************************
// Implementation Project Includes
// ***************************************************************************

#include "AbstractNodeView.h"
#include "ConnectorView.h"
#include "DormantNodeView.h"
#include "EavesdropperMain.h"
#include "InfoWindow.h"
#include "LiveNodeView.h"

// ***************************************************************************
// Implementation Constant Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Type Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Variable Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Variable Definitions
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions
// ***************************************************************************

// ===========================================================================
// Constructor
// ===========================================================================

InfoWindow::InfoWindow(BMediaRoster* mediaRoster) 
	: BWindow(BRect(390, 50, 630, 470), 
		"Information", 
		B_DOCUMENT_WINDOW,
		B_NOT_CLOSABLE | B_NOT_ZOOMABLE)
{
	// Initialize attributes.
	this->mediaRoster = mediaRoster;

	// -----------------------------------------------------------------------

	// Read the window's frame rectangle from the settings file.
	BFile file(SETTINGS_FILE, B_READ_ONLY);
	if (file.InitCheck() == B_OK)
	{
		if (file.Lock() == B_OK)
		{
			BRect rect;
			if (file.ReadAttr("InfoWindowFrame",
				B_RECT_TYPE,
				0,
				&rect,
				sizeof(BRect)) > 0)
			{
				MoveTo(rect.left,rect.top);
				ResizeTo(rect.Width(),rect.Height());
			}
      
			file.Unlock();
		}
	}

	// -----------------------------------------------------------------------
	
	infoView = new BTextView(
		BRect(0,
			0,
			Bounds().Width() - B_V_SCROLL_BAR_WIDTH,
			Bounds().Height() - B_H_SCROLL_BAR_HEIGHT),
		"infoView", 
		BRect(0,0,10000,10000),      // Always enough space to display text.
		B_FOLLOW_ALL_SIDES,
		B_WILL_DRAW);

	infoView->MakeEditable(false);
	infoView->MakeSelectable(false);
	infoView->SetStylable(true);
	infoView->SetWordWrap(false);
	infoView->SetAutoindent(false);

	AddChild(new BScrollView(
		"infoViewScroller", 
		infoView, 
		B_FOLLOW_ALL_SIDES, 
		B_FULL_UPDATE_ON_RESIZE, 
		true, 
		true,  
		B_PLAIN_BORDER));
}

// ===========================================================================
// FrameMoved
// ===========================================================================

void
InfoWindow::FrameMoved(BPoint screenPoint)
{
	// The little trick from FrameResized doesn't work from FrameMoved.
	SaveSettings();
}

// ===========================================================================
// FrameResized
// ===========================================================================

void
InfoWindow::FrameResized(float width, float height)
{ 
	BPoint point;
	uint32 buttons;
	infoView->GetMouse(&point,&buttons,false);
	if (!buttons)
	{
		SaveSettings();
	}
}

// ===========================================================================
// SelectNode
// ===========================================================================

void
InfoWindow::SelectNode(AbstractNodeView* node)
{
	if (selectedNode != 0)
	{
		selectedNode->SetSelected(false);
	}
	else if (selectedConnector != 0)
	{
		selectedConnector->SetSelected(false);
	}
	selectedConnector = 0;
	selectedNode = node;
	selectedNode->SetSelected(true);

	if (is_instance_of(selectedNode, LiveNodeView))
	{	
		ShowLiveNode((LiveNodeView*) selectedNode);
	}
	else
	{
		ShowDormantNode((DormantNodeView*) selectedNode);
	}
}

// ===========================================================================
// SelectConnector
// ===========================================================================

void
InfoWindow::SelectConnector(ConnectorView* connector)
{
	if (selectedNode != 0)
	{
		selectedNode->SetSelected(false);
	}
	else if (selectedConnector != 0)
	{
		selectedConnector->SetSelected(false);
	}
	selectedNode = 0;
	selectedConnector = connector;
	selectedConnector->SetSelected(true);

	ShowConnector(selectedConnector);
}

// ===========================================================================
// SelectNothing
// ===========================================================================

void
InfoWindow::SelectNothing()
{
	selectedNode = 0;
	selectedConnector = 0;
	
	if (Lock())
	{
		infoView->Delete(0, infoView->TextLength());
		Unlock();
	}
}
	 
// ***************************************************************************
// PROTECTED Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// PRIVATE Member Function Definitions
// ***************************************************************************

// ===========================================================================
// SaveSettings
// ===========================================================================

void
InfoWindow::SaveSettings(void)
{
	BFile file(SETTINGS_FILE, B_CREATE_FILE | B_WRITE_ONLY);
	if (file.InitCheck() == B_OK)
	{
		Lock();
		if (file.Lock() == B_OK)
		{
			BRect rect = Frame();
			file.WriteAttr("InfoWindowFrame",
				B_RECT_TYPE,
				0,
				&rect,
				sizeof(BRect));
      
			file.Sync();
			file.Unlock();
		}
		Unlock();
	}
}

// ===========================================================================
// ShowLiveNode
// ===========================================================================

void
InfoWindow::ShowLiveNode(LiveNodeView* liveNodeView)
{
	char string[1024];
	status_t result;
	
	live_node_info liveNodeInfo = liveNodeView->GetLiveNodeInfo();
	media_node node = liveNodeInfo.node;

	if (Lock())
	{
		// -------------------------------------------------------------------

		// Clear the text view.
		infoView->Delete(0, infoView->TextLength());

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Live Node\n\n");

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Name: ");
		
		infoView->SetFontAndColor(be_plain_font);
		infoView->Insert(liveNodeInfo.name);
		infoView->Insert("\n");	

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Node ID: ");

		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%ld\n", node.node);
		infoView->Insert(string);

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Port ID: ");

		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%ld\n", node.port);
		infoView->Insert(string);

		infoView->Insert("\n");

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Hint Point: ");

		infoView->SetFontAndColor(be_plain_font);
		BPoint hintPoint = liveNodeInfo.hint_point; 
		sprintf(string, "%.3f, %.3f\n", hintPoint.x, hintPoint.y);
		infoView->Insert(string);

		infoView->Insert("\n");

		// -------------------------------------------------------------------

		if (node.kind & B_BUFFER_PRODUCER)
		{
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Producer Latency: ");

			infoView->SetFontAndColor(be_plain_font);

			bigtime_t latency;
			result = mediaRoster->GetLatencyFor(node, &latency);

			if (result == B_OK)
			{
				sprintf(string, "%Ld\n", latency);
				infoView->Insert(string);
			}
			else
			{
				infoView->Insert("Error getting latency! ");
				infoView->Insert(strerror(result));
				infoView->Insert("\n");
			}
		}

   		// -------------------------------------------------------------------

		if (node.kind & B_TIME_SOURCE)
		{
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Time Source Start Latency: ");

			infoView->SetFontAndColor(be_plain_font);

			bigtime_t latency;
			result = mediaRoster->GetStartLatencyFor(node, &latency);

			if (result == B_OK)
			{
				sprintf(string, "%Ld\n", latency);
				infoView->Insert(string);
			}
			else
			{
				infoView->Insert("Error getting start latency! ");
				infoView->Insert(strerror(result));
				infoView->Insert("\n");
			}
		}

		if ((node.kind & B_BUFFER_PRODUCER) || (node.kind & B_TIME_SOURCE))
		{
			infoView->Insert("\n");
		}

   		// -------------------------------------------------------------------

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Node Kind: ");

		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "0x%lX\n", node.kind);
		infoView->Insert(string);
		WriteNodeKind(node.kind);

		infoView->Insert("\n");
		
   		// -------------------------------------------------------------------

		if (node.kind & B_BUFFER_CONSUMER)
		{
			BList* inputConnectors = liveNodeView->GetInputConnectors();
			
			int32 numFreeInputs = 0;
			int32 numConnectedInputs = 0;
			
			for (int32 t=0; t<inputConnectors->CountItems(); ++t)
			{
				ConnectorView* connectorView =
					(ConnectorView*) inputConnectors->ItemAt(t);
					
				if (connectorView->IsConnected())
				{
					++numConnectedInputs;
				}
				else
				{
					++numFreeInputs;
				}
			}

			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Number of connected inputs: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%ld\n", numConnectedInputs);
			infoView->Insert(string);
						
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Number of free inputs: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%ld\n", numFreeInputs);
			infoView->Insert(string);

			infoView->Insert("\n");
		}

   		// -------------------------------------------------------------------

		if (node.kind & B_BUFFER_PRODUCER)
		{
			BList* outputConnectors = liveNodeView->GetOutputConnectors();
			
			int32 numFreeOutputs = 0;
			int32 numConnectedOutputs = 0;
			
			for (int32 t=0; t<outputConnectors->CountItems(); ++t)
			{
				ConnectorView* connectorView =
					(ConnectorView*) outputConnectors->ItemAt(t);
					
				if (connectorView->IsConnected())
				{
					++numConnectedOutputs;
				}
				else
				{
					++numFreeOutputs;
				}
			}

			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Number of connected outputs: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%ld\n", numConnectedOutputs);
			infoView->Insert(string);
						
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Number of free outputs: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%ld\n", numFreeOutputs);
			infoView->Insert(string);

			infoView->Insert("\n");
		}

   		// -------------------------------------------------------------------

		BList* attributes = liveNodeView->GetAttributes();

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Number of attributes: ");

		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%ld\n", attributes->CountItems());
		infoView->Insert(string);

		if (attributes->CountItems() == 0)
		{
        	infoView->Insert("\n");
		}

		for (int32 t=0; t<attributes->CountItems(); ++t)
		{
			media_node_attribute* attribute =
				(media_node_attribute*) attributes->ItemAt(t);
				
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("What: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%lu\n", attribute->what);
			infoView->Insert(string);

			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Flags: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%lu\n", attribute->flags);
			infoView->Insert(string);

			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Data: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%Ld\n", attribute->data);
			infoView->Insert(string);

        	infoView->Insert("\n");
		}

   		// -------------------------------------------------------------------

		if (node.kind & B_FILE_INTERFACE)
		{
			BList* fileFormats = liveNodeView->GetFileFormats();
			
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Supported File Formats: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%ld\n", fileFormats->CountItems());
			infoView->Insert(string);

			infoView->Insert("\n");
                
			for (int32 t=0; t<fileFormats->CountItems(); ++t)
			{
				media_file_format* fileFormat =
					(media_file_format*) fileFormats->ItemAt(t);

				WriteMediaFileFormat(fileFormat);
			}
		}

   		// -------------------------------------------------------------------

		if (node.kind & B_FILE_INTERFACE)
		{
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Working with File: ");

			entry_ref fileRef;
			BMimeType fileMimeType;
			result = mediaRoster->GetRefFor(
				node,
				&fileRef,
				&fileMimeType);

			if (result == B_OK)
			{
				BEntry fileEntry(&fileRef);
				BPath path;
				fileEntry.GetPath(&path);

				infoView->SetFontAndColor(be_plain_font);
				infoView->Insert(path.Path());
				infoView->Insert("\n");

				infoView->SetFontAndColor(be_bold_font);
				infoView->Insert("Mime Type: ");

				const char* mimeTypeName = fileMimeType.Type();
				if (mimeTypeName != 0)
				{
					infoView->SetFontAndColor(be_plain_font);
					infoView->Insert(mimeTypeName);
					infoView->Insert("\n");
				}
				else
				{
					infoView->SetFontAndColor(be_plain_font);
					infoView->Insert("Could not get mime type\n");
				}
			}
			else
			{
				infoView->SetFontAndColor(be_plain_font);
				infoView->Insert("Error! ");
				infoView->Insert(strerror(result));
				infoView->Insert("\n\n");
			}
		}

		infoView->Insert("\n");

   		// -------------------------------------------------------------------

		/*!
		 * [TODO] InfoWindow::ShowLiveNode - Parameter webs on controllable
		 * nodes.
		 */

		Unlock();
	}
}

// ===========================================================================
// ShowDormantNode
// ===========================================================================

void
InfoWindow::ShowDormantNode(DormantNodeView* dormantNodeView)
{
	char string[1024];
	status_t result;
	
	dormant_node_info dormantNodeInfo =
		dormantNodeView->GetDormantNodeInfo();
	dormant_flavor_info dormantFlavorInfo =
		dormantNodeView->GetDormantFlavorInfo();

	if (Lock())
	{
		// -------------------------------------------------------------------

		// Clear the text view.
		infoView->Delete(0, infoView->TextLength());

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Dormant Node\n\n");

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Name: ");
		
		infoView->SetFontAndColor(be_plain_font);
		infoView->Insert(dormantNodeInfo.name);
		infoView->Insert("\n");

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Add-On ID: ");

		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%ld\n", dormantNodeInfo.addon);
		infoView->Insert(string);

		infoView->Insert("\n");

		// -------------------------------------------------------------------

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Flavor ID: ");

		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%ld\n", dormantNodeInfo.flavor_id);
		infoView->Insert(string);

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Flavor Name: ");

		infoView->SetFontAndColor(be_plain_font);
		infoView->Insert(dormantFlavorInfo.name);
		infoView->Insert("\n");

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Flavor Info: ");

		infoView->SetFontAndColor(be_plain_font);
		infoView->Insert(dormantFlavorInfo.info);
		infoView->Insert("\n");

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Internal ID: ");

		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%ld\n", dormantFlavorInfo.internal_id);
		infoView->Insert(string);

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Possible Count: ");

		if (dormantFlavorInfo.possible_count != 0)
		{
			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%ld\n", dormantFlavorInfo.possible_count);
			infoView->Insert(string);
		}
		else
		{
			infoView->SetFontAndColor(be_plain_font);
			infoView->Insert("Any number");
		}

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Flags: ");

		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "0x%lx\n", dormantFlavorInfo.flavor_flags);
		infoView->Insert(string);

		if (dormantFlavorInfo.flavor_flags & B_FLAVOR_IS_GLOBAL)
		{
			infoView->Insert("B_FLAVOR_IS_GLOBAL\n");
		}
		if (dormantFlavorInfo.flavor_flags & B_FLAVOR_IS_LOCAL)
		{
			infoView->Insert("B_FLAVOR_IS_LOCAL\n");
		}

		infoView->Insert("\n");
				
		// -------------------------------------------------------------------

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Node Kind: ");

		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "0x%LX\n", dormantFlavorInfo.kinds);
		infoView->Insert(string);
		WriteNodeKind(dormantFlavorInfo.kinds);

		infoView->Insert("\n");

		// -------------------------------------------------------------------

		if (dormantFlavorInfo.kinds & B_BUFFER_CONSUMER)
		{
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Supported Input Formats: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%ld\n", dormantFlavorInfo.in_format_count);
			infoView->Insert(string);

			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Input Format Flags: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%lu\n", dormantFlavorInfo.in_format_flags);
			infoView->Insert(string);

			for (int32 t=0; t<dormantFlavorInfo.in_format_count; ++t)
			{
				WriteMediaFormatSummary(dormantFlavorInfo.in_formats[t]);
			}

			infoView->Insert("\n");			
		}
		
		// -------------------------------------------------------------------

		if (dormantFlavorInfo.kinds & B_BUFFER_PRODUCER)
		{
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Supported Output Formats: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%ld\n", dormantFlavorInfo.out_format_count);
			infoView->Insert(string);

			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Output Format Flags: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%lu\n", dormantFlavorInfo.out_format_flags);
			infoView->Insert(string);

			for (int32 t=0; t<dormantFlavorInfo.out_format_count; ++t)
			{
				WriteMediaFormatSummary(dormantFlavorInfo.out_formats[t]);
			}

			infoView->Insert("\n");							
		}

		// -------------------------------------------------------------------

		if (dormantFlavorInfo.kinds & B_FILE_INTERFACE)
		{
			BList* fileFormats = dormantNodeView->GetInputFileFormats();
		
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Supported File Formats For Reading: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%ld\n", fileFormats->CountItems());
			infoView->Insert(string);

			infoView->Insert("\n");
                
			for (int32 t=0; t<fileFormats->CountItems(); ++t)
			{
				media_file_format* fileFormat =
					(media_file_format*) fileFormats->ItemAt(t);

				WriteMediaFileFormat(fileFormat);
			}				
		}

		// -------------------------------------------------------------------

		if (dormantFlavorInfo.kinds & B_FILE_INTERFACE)
		{
			BList* fileFormats = dormantNodeView->GetOutputFileFormats();
		
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Supported File Formats For Writing: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%ld\n", fileFormats->CountItems());
			infoView->Insert(string);

			infoView->Insert("\n");
                
			for (int32 t=0; t<fileFormats->CountItems(); ++t)
			{
				media_file_format* fileFormat =
					(media_file_format*) fileFormats->ItemAt(t);

				WriteMediaFileFormat(fileFormat);
			}				
		}

		// -------------------------------------------------------------------

		Unlock();
	}
}

// ===========================================================================
// ShowConnector
// ===========================================================================

void
InfoWindow::ShowConnector(ConnectorView* connectorView)
{
	char string[1024];

	if (Lock())
	{
		// Clear the text view.
		infoView->Delete(0, infoView->TextLength());

		if (connectorView->IsInput())
		{
			media_input* mediaInput = connectorView->GetInput();
			
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Input\n\n");

			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Name: ");
		
			infoView->SetFontAndColor(be_plain_font);
			infoView->Insert(mediaInput->name);
			infoView->Insert("\n");	

			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Owner Node ID: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%ld\n", mediaInput->node.node);
			infoView->Insert(string);

			infoView->Insert("\n");	
			
			WriteMediaSource(mediaInput->source);
			WriteMediaDestination(mediaInput->destination);
			WriteMediaFormat(mediaInput->format);
		}
		else
		{
			media_output* mediaOutput = connectorView->GetOutput();
					
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Output\n\n");
			
			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Name: ");
		
			infoView->SetFontAndColor(be_plain_font);
			infoView->Insert(mediaOutput->name);
			infoView->Insert("\n");	

			infoView->SetFontAndColor(be_bold_font);
			infoView->Insert("Owner Node ID: ");

			infoView->SetFontAndColor(be_plain_font);
			sprintf(string, "%ld\n", mediaOutput->node.node);
			infoView->Insert(string);			

			infoView->Insert("\n");	

			WriteMediaSource(mediaOutput->source);
			WriteMediaDestination(mediaOutput->destination);
			WriteMediaFormat(mediaOutput->format);
		}
		Unlock();
	}
}

// ===========================================================================
// WriteNodeKind
// ===========================================================================

void
InfoWindow::WriteNodeKind(uint32 kind)
{
	if (kind & B_BUFFER_PRODUCER)
	{
		infoView->Insert("B_BUFFER_PRODUCER\n");
	}        
	if (kind & B_BUFFER_CONSUMER)
	{
    	infoView->Insert("B_BUFFER_CONSUMER\n");
	}        
	if (kind & B_TIME_SOURCE)
	{
		infoView->Insert("B_TIME_SOURCE\n");
	}        
	if (kind & B_CONTROLLABLE)
	{
		infoView->Insert("B_CONTROLLABLE\n");
	}        
	if (kind & B_FILE_INTERFACE)
	{
		infoView->Insert("B_FILE_INTERFACE\n");
	}        
	if (kind & B_ENTITY_INTERFACE)
	{
		infoView->Insert("B_ENTITY_INTERFACE\n");
	}        
	if (kind & B_PHYSICAL_INPUT)
	{
		infoView->Insert("B_PHYSICAL_INPUT\n");
	}        
	if (kind & B_PHYSICAL_OUTPUT)
	{
		infoView->Insert("B_PHYSICAL_OUTPUT\n");
	}        
	if (kind & B_SYSTEM_MIXER)
	{
		infoView->Insert("B_SYSTEM_MIXER\n");
	}        
}

// ===========================================================================
// WriteMediaFileFormat
// ===========================================================================

void 
InfoWindow::WriteMediaFileFormat(media_file_format* fileFormat)
{
	char string[1024];

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("File Format Internal ID: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu\n", fileFormat->id.internal_id);
	infoView->Insert(string);

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("File Format Node ID: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%Lu\n", fileFormat->id.node);
	infoView->Insert(string);
	
	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("File Format Device ID: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu\n", fileFormat->id.device);
	infoView->Insert(string);
		
	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("File Format Capabilities: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "0x%lx\n", fileFormat->capabilities);
	infoView->Insert(string);	
	
	if (fileFormat->capabilities & media_file_format::B_READABLE)
	{
		infoView->Insert("B_READABLE\n");
	}        
	if (fileFormat->capabilities & media_file_format::B_WRITABLE)
	{
		infoView->Insert("B_WRITABLE\n");
	}        
	if (fileFormat->capabilities & media_file_format::B_PERFECTLY_SEEKABLE)
	{
		infoView->Insert("B_PERFECTLY_SEEKABLE\n");
	}        
	if (fileFormat->capabilities & media_file_format::B_IMPERFECTLY_SEEKABLE)
	{
		infoView->Insert("B_IMPERFECTLY_SEEKABLE\n");
	}        
	if (fileFormat->capabilities & media_file_format::B_KNOWS_RAW_VIDEO)
	{
		infoView->Insert("B_KNOWS_RAW_VIDEO\n");
	}        
	if (fileFormat->capabilities & media_file_format::B_KNOWS_RAW_AUDIO)
	{
		infoView->Insert("B_KNOWS_RAW_AUDIO\n");
	}        
	if (fileFormat->capabilities & media_file_format::B_KNOWS_MIDI)
	{
		infoView->Insert("B_KNOWS_MIDI\n");
	}        
	if (fileFormat->capabilities & media_file_format::B_KNOWS_ENCODED_VIDEO)
	{
		infoView->Insert("B_KNOWS_ENCODED_VIDEO\n");
	}        
	if (fileFormat->capabilities & media_file_format::B_KNOWS_ENCODED_AUDIO)
	{
		infoView->Insert("B_KNOWS_ENCODED_AUDIO\n");
	}        
	if (fileFormat->capabilities & media_file_format::B_KNOWS_OTHER)
	{
		infoView->Insert("B_KNOWS_OTHER\n");
	}        
	if (fileFormat->capabilities & media_file_format::B_KNOWS_ANYTHING)
	{
		infoView->Insert("B_KNOWS_ANYTHING\n");
	}        

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Media Format Family: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%ld\n", fileFormat->family);
	infoView->Insert(string);

	if (fileFormat->family == B_ANY_FORMAT_FAMILY)
	{
		infoView->Insert("B_ANY_FORMAT_FAMILY\n");
	}
	else if (fileFormat->family == B_BEOS_FORMAT_FAMILY)
	{
		infoView->Insert("B_BEOS_FORMAT_FAMILY\n");
	}
	else if (fileFormat->family == B_QUICKTIME_FORMAT_FAMILY)
	{
		infoView->Insert("B_QUICKTIME_FORMAT_FAMILY\n");
	}
	else if (fileFormat->family == B_AVI_FORMAT_FAMILY)
	{
		infoView->Insert("B_AVI_FORMAT_FAMILY\n");
	}
	else if (fileFormat->family == B_ASF_FORMAT_FAMILY)
	{
		infoView->Insert("B_ASF_FORMAT_FAMILY\n");
	}
	else if (fileFormat->family == B_MPEG_FORMAT_FAMILY)
	{
		infoView->Insert("B_MPEG_FORMAT_FAMILY\n");
	}
	else if (fileFormat->family == B_WAV_FORMAT_FAMILY)
	{
		infoView->Insert("B_WAV_FORMAT_FAMILY\n");
	}
	else if (fileFormat->family == B_AIFF_FORMAT_FAMILY)
	{
		infoView->Insert("B_AIFF_FORMAT_FAMILY\n");
	}
	else if (fileFormat->family == B_MISC_FORMAT_FAMILY)
	{
		infoView->Insert("B_MISC_FORMAT_FAMILY\n");
	}

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("File Format Version: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%ld\n", fileFormat->version);
	infoView->Insert(string);
	
	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("File Format Mime Type: ");
		
	infoView->SetFontAndColor(be_plain_font);
	infoView->Insert(fileFormat->mime_type);
	infoView->Insert("\n");

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("File Format Pretty Name: ");
		
	infoView->SetFontAndColor(be_plain_font);
	infoView->Insert(fileFormat->pretty_name);
	infoView->Insert("\n");

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("File Format Short Name: ");
		
	infoView->SetFontAndColor(be_plain_font);
	infoView->Insert(fileFormat->short_name);
	infoView->Insert("\n");

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("File Format File Extension: ");
		
	infoView->SetFontAndColor(be_plain_font);
	infoView->Insert(fileFormat->file_extension);
	infoView->Insert("\n");

	infoView->Insert("\n");
}

// ===========================================================================
// WriteMediaSource
// ===========================================================================

void
InfoWindow::WriteMediaSource(media_source mediaSource)
{
	char string[1024];

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Source ID: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%ld\n", mediaSource.id);
	infoView->Insert(string);			
			
	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Source Port ID: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%ld\n", mediaSource.port);
	infoView->Insert(string);			

	infoView->Insert("\n");
}

// ===========================================================================
// WriteMediaDestination
// ===========================================================================

void
InfoWindow::WriteMediaDestination(media_destination mediaDestination)
{
	char string[1024];

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Destination ID: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%ld\n", mediaDestination.id);
	infoView->Insert(string);			
			
	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Destination Port ID: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%ld\n", mediaDestination.port);
	infoView->Insert(string);			
	
	infoView->Insert("\n");
}

// ===========================================================================
// WriteMediaFormatSummary
// ===========================================================================

void
InfoWindow::WriteMediaFormatSummary(media_format mediaFormat)
{
	char string[1024];

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Format Type: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "0x%lx (", mediaFormat.type);
	infoView->Insert(string);			
	WriteMediaType(mediaFormat.type);
	infoView->Insert(")\n");
}

// ===========================================================================
// WriteMediaFormat
// ===========================================================================

void
InfoWindow::WriteMediaFormat(media_format mediaFormat)
{
	char string[1024];

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Format Type: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "0x%lx (", mediaFormat.type);
	infoView->Insert(string);			
	WriteMediaType(mediaFormat.type);
	infoView->Insert(")\n");

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("User Data Type Code: ");
		
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lx \'%1c%1c%1c%1c\' ", 
		mediaFormat.user_data_type,
		(char) (mediaFormat.user_data_type >> 24) & 0xFF,
		(char) (mediaFormat.user_data_type >> 16) & 0xFF,
		(char) (mediaFormat.user_data_type >> 8) & 0xFF,
		(char) (mediaFormat.user_data_type) & 0xFF);
	infoView->Insert(string);
	WriteTypeCode(mediaFormat.user_data_type);

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("User Data: ");
		
	infoView->SetFontAndColor(be_plain_font);
	for (int32 t=0; t<64; ++t)
	{
		sprintf(string, " %02x", mediaFormat.user_data[t]);
		infoView->Insert(string);
	}

	infoView->Insert("\n");

	/*if (string_for_format(mediaFormat, string, 1024))
	{
		BBG_WRITE(("string_for_format ok!"))
		infoView->Insert(string);
		infoView->Insert("\n");
	}*/

	// -----------------------------------------------------------------------

	if (mediaFormat.type == B_MEDIA_RAW_AUDIO)
	{
		WriteRawAudioFormat(mediaFormat.u.raw_audio);
	}

	// -----------------------------------------------------------------------

	else if (mediaFormat.type == B_MEDIA_RAW_VIDEO)
	{
		WriteRawVideoFormat(mediaFormat.u.raw_video);
	}

	// -----------------------------------------------------------------------

	else if (mediaFormat.type == B_MEDIA_MULTISTREAM)
	{
		media_multistream_format format = mediaFormat.u.multistream;

		/*!
		 * [TODO] InfoWindow::WriteMediaFormat -- Implement
		 * B_MEDIA_MULTISTREAM. I don't think I have a node to test it with.
		 */
	}

	// -----------------------------------------------------------------------

	else if (mediaFormat.type == B_MEDIA_ENCODED_AUDIO)
	{
		media_encoded_audio_format format = mediaFormat.u.encoded_audio;
		
		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Bit Rate: ");
		
		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%f\n", format.bit_rate);
		infoView->Insert(string);			

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Frame Size: ");
		
		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%lu\n", format.frame_size);
		infoView->Insert(string);
		
		WriteRawAudioFormat(format.output);
	}

	// -----------------------------------------------------------------------

	else if (mediaFormat.type == B_MEDIA_ENCODED_VIDEO)
	{
		media_encoded_video_format format = mediaFormat.u.encoded_video;

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Average Bit Rate: ");
		
		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%f\n", format.avg_bit_rate);
		infoView->Insert(string);			

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Maximum Bit Rate: ");
		
		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%f\n", format.max_bit_rate);
		infoView->Insert(string);
		
		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Frame Size: ");
		
		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%lu\n", format.frame_size);
		infoView->Insert(string);			

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Forward History: ");
		
		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%ld\n", format.forward_history);
		infoView->Insert(string);

		infoView->SetFontAndColor(be_bold_font);
		infoView->Insert("Backward History: ");
		
		infoView->SetFontAndColor(be_plain_font);
		sprintf(string, "%ld\n", format.backward_history);
		infoView->Insert(string);
		
		WriteRawVideoFormat(format.output);
	}
}

// ===========================================================================
// WriteMediaType
// ===========================================================================

void
InfoWindow::WriteMediaType(media_type mediaType)
{
	if (mediaType == B_MEDIA_NO_TYPE)
	{
		infoView->Insert("B_MEDIA_NO_TYPE");
		return;
	}
	if (mediaType == B_MEDIA_UNKNOWN_TYPE)
	{
		infoView->Insert("B_MEDIA_UNKNOWN_TYPE");
		return;
	}
	if (mediaType == B_MEDIA_RAW_AUDIO)
	{
		infoView->Insert("B_MEDIA_RAW_AUDIO");
		return;
	}
	if (mediaType == B_MEDIA_RAW_VIDEO)
	{
		infoView->Insert("B_MEDIA_RAW_VIDEO");
		return;
	}
	if (mediaType == B_MEDIA_VBL)
	{
		infoView->Insert("B_MEDIA_VBL");
		return;
	}
	if (mediaType == B_MEDIA_TIMECODE)
	{
		infoView->Insert("B_MEDIA_TIMECODE");
		return;
	}
	if (mediaType == B_MEDIA_MIDI)
	{
		infoView->Insert("B_MEDIA_MIDI");
		return;
	}
	if (mediaType == B_MEDIA_TEXT)
	{
		infoView->Insert("B_MEDIA_TEXT");
		return;
	}
	if (mediaType == B_MEDIA_HTML)
	{
		infoView->Insert("B_MEDIA_HTML");
		return;
	}
	if (mediaType == B_MEDIA_MULTISTREAM)
	{
		infoView->Insert("B_MEDIA_MULTISTREAM");
		return;
	}
	if (mediaType == B_MEDIA_PARAMETERS)
	{
		infoView->Insert("B_MEDIA_PARAMETERS");
		return;
	}
	if (mediaType == B_MEDIA_ENCODED_AUDIO)
	{
		infoView->Insert("B_MEDIA_ENCODED_AUDIO");
		return;
	}
	if (mediaType == B_MEDIA_ENCODED_VIDEO)
	{
		infoView->Insert("B_MEDIA_ENCODED_VIDEO");
		return;
	}
	if ((mediaType >= B_MEDIA_PRIVATE) 
	&&  (mediaType < B_MEDIA_FIRST_USER_TYPE))
	{
		infoView->Insert("B_MEDIA_PRIVATE");
		return;
	}
	if (mediaType >= B_MEDIA_FIRST_USER_TYPE)
	{
		infoView->Insert("Experimental user-defined type");
		return;
	}
	infoView->Insert("Unknown media type");
}

// ===========================================================================
// WriteTypeCode
// ===========================================================================

void 
InfoWindow::WriteTypeCode(type_code typeCode)
{
	if (typeCode == B_ANY_TYPE)
	{
		infoView->Insert("(B_ANY_TYPE)\n");
		return;
	}
	if (typeCode == B_ASCII_TYPE)
	{
		infoView->Insert("(B_ASCII_TYPE)\n");
		return;
	}
	if (typeCode == B_BOOL_TYPE)
	{
		infoView->Insert("(B_BOOL_TYPE)\n");
		return;
	}
	if (typeCode == B_CHAR_TYPE)
	{
		infoView->Insert("(B_CHAR_TYPE)\n");
		return;
	}
	if (typeCode == B_COLOR_8_BIT_TYPE)
	{
		infoView->Insert("(B_COLOR_8_BIT_TYPE)\n");
		return;
	}
	if (typeCode == B_DOUBLE_TYPE)
	{
		infoView->Insert("(B_DOUBLE_TYPE)\n");
		return;
	}
	if (typeCode == B_FLOAT_TYPE)
	{
		infoView->Insert("(B_FLOAT_TYPE)\n");
		return;
	}
	if (typeCode == B_GRAYSCALE_8_BIT_TYPE)
	{
		infoView->Insert("(B_GRAYSCALE_8_BIT_TYPE)\n");
		return;
	}
	if (typeCode == B_INT64_TYPE)
	{
		infoView->Insert("(B_INT64_TYPE)\n");
		return;
	}
	if (typeCode == B_INT32_TYPE)
	{
		infoView->Insert("(B_INT32_TYPE)\n");
		return;
	}
	if (typeCode == B_INT16_TYPE)
	{
		infoView->Insert("(B_INT16_TYPE)\n");
		return;
	}
	if (typeCode == B_INT8_TYPE)
	{
		infoView->Insert("(B_INT8_TYPE)\n");
		return;
	}
	if (typeCode == B_MESSAGE_TYPE)
	{
		infoView->Insert("(B_MESSAGE_TYPE)\n");
		return;
	}
	if (typeCode == B_MESSENGER_TYPE)
	{
		infoView->Insert("(B_MESSENGER_TYPE)\n");
		return;
	}
	if (typeCode == B_MIME_TYPE)
	{
		infoView->Insert("(B_MIME_TYPE)\n");
		return;
	}
	if (typeCode == B_MONOCHROME_1_BIT_TYPE)
	{
		infoView->Insert("(B_MONOCHROME_1_BIT_TYPE)\n");
		return;
	}
	if (typeCode == B_OBJECT_TYPE)
	{
		infoView->Insert("(B_OBJECT_TYPE)\n");
		return;
	}
	if (typeCode == B_OFF_T_TYPE)
	{
		infoView->Insert("(B_OFF_T_TYPE)\n");
		return;
	}
	if (typeCode == B_PATTERN_TYPE)
	{
		infoView->Insert("(B_PATTERN_TYPE)\n");
		return;
	}
	if (typeCode == B_POINTER_TYPE)
	{
		infoView->Insert("(B_POINTER_TYPE)\n");
		return;
	}
	if (typeCode == B_POINT_TYPE)
	{
		infoView->Insert("(B_POINT_TYPE)\n");
		return;
	}
	if (typeCode == B_RAW_TYPE)
	{
		infoView->Insert("(B_RAW_TYPE)\n");
		return;
	}
	if (typeCode == B_RECT_TYPE)
	{
		infoView->Insert("(B_RECT_TYPE)\n");
		return;
	}
	if (typeCode == B_REF_TYPE)
	{
		infoView->Insert("(B_REF_TYPE)\n");
		return;
	}
	if (typeCode == B_RGB_32_BIT_TYPE)
	{
		infoView->Insert("(B_RGB_32_BIT_TYPE)\n");
		return;
	}
	if (typeCode == B_RGB_COLOR_TYPE)
	{
		infoView->Insert("(B_RGB_COLOR_TYPE)\n");
		return;
	}
	if (typeCode == B_SIZE_T_TYPE)
	{
		infoView->Insert("(B_SIZE_T_TYPE)\n");
		return;
	}
	if (typeCode == B_SSIZE_T_TYPE)
	{
		infoView->Insert("(B_SSIZE_T_TYPE)\n");
		return;
	}
	if (typeCode == B_STRING_TYPE)
	{
		infoView->Insert("(B_STRING_TYPE)\n");
		return;
	}
	if (typeCode == B_TIME_TYPE)
	{
		infoView->Insert("(B_TIME_TYPE)\n");
		return;
	}
	if (typeCode == B_UINT64_TYPE)
	{
		infoView->Insert("(B_UINT64_TYPE)\n");
		return;
	} 
	if (typeCode == B_UINT32_TYPE)
	{
		infoView->Insert("(B_UINT32_TYPE)\n");
		return;
	} 
	if (typeCode == B_UINT16_TYPE)
	{
		infoView->Insert("(B_UINT16_TYPE)\n");
		return;
	} 
	if (typeCode == B_UINT8_TYPE)
	{
		infoView->Insert("(B_UINT8_TYPE)\n");
		return;
	} 
	if (typeCode == B_MEDIA_PARAMETER_TYPE)
	{
		infoView->Insert("(B_MEDIA_PARAMETER_TYPE)\n");
		return;
	} 
	if (typeCode == B_MEDIA_PARAMETER_WEB_TYPE)
	{
		infoView->Insert("(B_MEDIA_PARAMETER_WEB_TYPE)\n");
		return;
	} 
	if (typeCode == B_MEDIA_PARAMETER_GROUP_TYPE)
	{
		infoView->Insert("(B_MEDIA_PARAMETER_GROUP_TYPE)\n");
		return;
	} 
	infoView->Insert("(Unknown type code)\n");
}

// ===========================================================================
// WriteColorSpace
// ===========================================================================

void
InfoWindow::WriteColorSpace(color_space colorSpace)
{
	if (colorSpace == B_RGB32)
	{
		infoView->Insert("(B_RGB32)\n");
		return;
	}
	if (colorSpace == B_RGBA32)
	{
		infoView->Insert("(B_RGBA32)\n");
		return;
	}
	if (colorSpace == B_RGB24)
	{
		infoView->Insert("(B_RGB24)\n");
		return;
	}
	if (colorSpace == B_RGB16)
	{
		infoView->Insert("(B_RGB16)\n");
		return;
	}
	if (colorSpace == B_RGB15)
	{
		infoView->Insert("(B_RGB15)\n");
		return;
	}
	if (colorSpace == B_RGBA15)
	{
		infoView->Insert("(B_RGBA15)\n");
		return;
	}
	if (colorSpace == B_CMAP8)
	{
		infoView->Insert("(B_CMAP8)\n");
		return;
	}
	if (colorSpace == B_GRAY8)
	{
		infoView->Insert("(B_GRAY8)\n");
		return;
	}
	if (colorSpace == B_GRAY1)
	{
		infoView->Insert("(B_GRAY1)\n");
		return;
	}
	if (colorSpace == B_RGB32_BIG)
	{
		infoView->Insert("(B_RGB32_BIG)\n");
		return;
	}
	if (colorSpace == B_RGBA32_BIG)
	{
		infoView->Insert("(B_RGBA32_BIG)\n");
		return;
	}
	if (colorSpace == B_RGB24_BIG)
	{
		infoView->Insert("(B_RGB24_BIG)\n");
		return;
	}
	if (colorSpace == B_RGB16_BIG)
	{
		infoView->Insert("(B_RGB16_BIG)\n");
		return;
	}
	if (colorSpace == B_RGB15_BIG)
	{
		infoView->Insert("(B_RGB15_BIG)\n");
		return;
	}
	if (colorSpace == B_RGBA15_BIG)
	{
		infoView->Insert("(B_RGBA15_BIG)\n");
		return;
	}
	if (colorSpace == B_YCbCr422)
	{
		infoView->Insert("(B_YCbCr422)\n");
		return;
	}
	if (colorSpace == B_YCbCr411)
	{
		infoView->Insert("(B_YCbCr411)\n");
		return;
	}
	if (colorSpace == B_YCbCr444)
	{
		infoView->Insert("(B_YCbCr444)\n");
		return;
	}
	if (colorSpace == B_YCbCr420)
	{
		infoView->Insert("(B_YCbCr420)\n");
		return;
	}
	if (colorSpace == B_YUV422)
	{
		infoView->Insert("(B_YUV422)\n");
		return;
	}
	if (colorSpace == B_YUV411)
	{
		infoView->Insert("(B_YUV411)\n");
		return;
	}
	if (colorSpace == B_YUV444)
	{
		infoView->Insert("(B_YUV444)\n");
		return;
	}
	if (colorSpace == B_YUV420)
	{
		infoView->Insert("(B_YUV420)\n");
		return;
	}
	if (colorSpace == B_YUV9)
	{
		infoView->Insert("(B_YUV9)\n");
		return;
	}
	if (colorSpace == B_YUV12)
	{
		infoView->Insert("(B_YUV12)\n");
		return;
	}
	if (colorSpace == B_UVL24)
	{
		infoView->Insert("(B_UVL24)\n");
		return;
	}
	if (colorSpace == B_UVL32)
	{
		infoView->Insert("(B_UVL32)\n");
		return;
	}
	if (colorSpace == B_UVLA32)
	{
		infoView->Insert("(B_UVLA32)\n");
		return;
	}
	if (colorSpace == B_LAB24)
	{
		infoView->Insert("(B_LAB24)\n");
		return;
	}
	if (colorSpace == B_LAB32)
	{
		infoView->Insert("(B_LAB32)\n");
		return;
	}
	if (colorSpace == B_LABA32)
	{
		infoView->Insert("(B_LABA32)\n");
		return;
	}
	if (colorSpace == B_HSI24)
	{
		infoView->Insert("(B_HSI24)\n");
		return;
	}
	if (colorSpace == B_HSI32)
	{
		infoView->Insert("(B_HSI32)\n");
		return;
	}
	if (colorSpace == B_HSIA32)
	{
		infoView->Insert("(B_HSIA32)\n");
		return;
	}
	if (colorSpace == B_HSV24)
	{
		infoView->Insert("(B_HSV24)\n");
		return;
	}
	if (colorSpace == B_HSV32)
	{
		infoView->Insert("(B_HSV32)\n");
		return;
	}
	if (colorSpace == B_HSVA32)
	{
		infoView->Insert("(B_HSVA32)\n");
		return;
	}
	if (colorSpace == B_HLS24)
	{
		infoView->Insert("(B_HLS24)\n");
		return;
	}
	if (colorSpace == B_HLS32)
	{
		infoView->Insert("(B_HLS32)\n");
		return;
	}
	if (colorSpace == B_HLSA32)
	{
		infoView->Insert("(B_HLSA32)\n");
		return;
	}
	if (colorSpace == B_CMY24)
	{
		infoView->Insert("(B_CMY24)\n");
		return;
	}
	if (colorSpace == B_CMY32)
	{
		infoView->Insert("(B_CMY32)\n");
		return;
	}
	if (colorSpace == B_CMYA32)
	{
		infoView->Insert("(B_CMYA32)\n");
		return;
	}
	if (colorSpace == B_CMYK32)
	{
		infoView->Insert("(B_CMYK32)\n");
		return;
	}
	infoView->Insert("(Unknown color space)\n");
}

// ===========================================================================
// WriteRawAudioFormat
// ===========================================================================

void
InfoWindow::WriteRawAudioFormat(media_raw_audio_format format)
{
	char string[1024];
	
	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Frame Rate: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%f\n", format.frame_rate);
	infoView->Insert(string);			

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Channel Count: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu\n", format.channel_count);
	infoView->Insert(string);			

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Format: ");

	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu ", format.format);
	infoView->Insert(string);			
	
	if (format.format == media_raw_audio_format::B_AUDIO_UCHAR)
	{
		infoView->Insert("(B_AUDIO_UCHAR)\n");
	}
	else if (format.format == media_raw_audio_format::B_AUDIO_SHORT)
	{
		infoView->Insert("(B_AUDIO_SHORT)\n");
	}
	else if (format.format == media_raw_audio_format::B_AUDIO_FLOAT)
	{
		infoView->Insert("(B_AUDIO_FLOAT)\n");
	}
	else if (format.format == media_raw_audio_format::B_AUDIO_INT)
	{
		infoView->Insert("(B_AUDIO_INT)\n");
	}
	else
	{
		infoView->Insert("(Unknown format)\n");
	}
	
	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Byte Order: ");

	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu ", format.byte_order);
	infoView->Insert(string);							

	if (format.byte_order == B_MEDIA_BIG_ENDIAN)
	{
		infoView->Insert("(Big endian)\n");
	}
	else if (format.byte_order == B_MEDIA_LITTLE_ENDIAN)
	{
		infoView->Insert("(Little endian)\n");
	}
	else
	{
		infoView->Insert("(Unknown byte order)\n");
	}

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Buffer Size: ");

	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu\n", format.buffer_size);
	infoView->Insert(string);
}

// ===========================================================================
// WriteRawVideoFormat
// ===========================================================================

void
InfoWindow::WriteRawVideoFormat(media_raw_video_format format)
{
	char string[1024];
	
	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Field Rate: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%f\n", format.field_rate);
	infoView->Insert(string);			

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Fields per Frame: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu\n", format.interlace);
	infoView->Insert(string);			

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("First Active Scan Line: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu\n", format.first_active);
	infoView->Insert(string);

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Last Active Scan Line: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu\n", format.last_active);
	infoView->Insert(string);

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Orientation: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu ", format.orientation);
	infoView->Insert(string);

	if (format.orientation == B_VIDEO_TOP_LEFT_RIGHT)
	{
		infoView->Insert("(B_VIDEO_TOP_LEFT_RIGHT)\n");
	}
	else if (format.orientation == B_VIDEO_BOTTOM_LEFT_RIGHT)
	{
		infoView->Insert("(B_VIDEO_BOTTOM_LEFT_RIGHT)\n");
	}
	else
	{
		infoView->Insert("(Unknown orientation)\n");
	}
	
	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Aspect Ratio: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu/%lu\n",
		format.pixel_width_aspect,
		format.pixel_height_aspect);
	infoView->Insert(string);

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Color Space: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "0x%lx ", format.display.format);
	infoView->Insert(string);
	WriteColorSpace(format.display.format);

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Line Width: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu\n", format.display.line_width);
	infoView->Insert(string);

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Line Count: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu\n", format.display.line_count);
	infoView->Insert(string);

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Bytes per Row: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu\n", format.display.bytes_per_row);
	infoView->Insert(string);

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Pixel Offset: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu\n", format.display.pixel_offset);
	infoView->Insert(string);

	infoView->SetFontAndColor(be_bold_font);
	infoView->Insert("Line Offset: ");
	
	infoView->SetFontAndColor(be_plain_font);
	sprintf(string, "%lu\n", format.display.line_offset);
	infoView->Insert(string);
}
	
// ***************************************************************************
// Helper Function Definitions
// ***************************************************************************

// ***************************************************************************
// Global Function Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Definitions
// ***************************************************************************
